
#include "aes.h"

char byteIn; // tmp byte read from serial

//PINS FOR TRIGGER
unsigned pinTrigger = 19; //A5; on ULB board

unsigned char key [Nrows][Ncols];
unsigned char workState [Nrows][Ncols];
unsigned char state[Nrows][Ncols];
unsigned char shuffle;
unsigned int  longShuffle;

void setup(){
  Serial.begin(9600); //setup communication
  randomSeed(1);  //setup random numbers
  
  //setup trigger pins
  pinMode(pinTrigger, OUTPUT);
  digitalWrite(pinTrigger, LOW);
  
  //setup key (will not change during execution)
  // KEY : 0x 33BBAA55 00010203 0E70766E FCFDFEFF
  
  key[0][0] = 0x33;  key[0][1] = 0x00;  key[0][2] = 0x0E;  key[0][3] = 0xFC;
  key[1][0] = 0xBB;  key[1][1] = 0x01;  key[1][2] = 0x70;  key[1][3] = 0xFD;
  key[2][0] = 0xAA;  key[2][1] = 0x02;  key[2][2] = 0x76;  key[2][3] = 0xFE;
  key[3][0] = 0x55;  key[3][1] = 0x03;  key[3][2] = 0x6E;  key[3][3] = 0xFF;
  
  speedTest();
  
}

void loop(){
  if (Serial.available() > 0){
    byteIn = Serial.read(); //read command byte
    Serial.println(byteIn);
    if (byteIn == 'c'){
      initState(state, workState);
      digitalWrite(pinTrigger, HIGH);
      cipher(workState, key);
      printStateHex(workState);
      digitalWrite(pinTrigger, LOW);
      
      initState(state, workState);
      cipher_shuffle1(workState, key, shuffle&0x01 ); //keep 1 LSB
      printStateHex(workState);
      initState(state, workState);
      cipher_shuffle2(workState, key, shuffle&0x03 ); // keep 2 LSBs
      printStateHex(workState);
      initState(state, workState);
      cipher_shuffle3(workState, key, shuffle&0x07 ); // keep 3 LSBs
      printStateHex(workState);
      initState(state, workState);
      cipher_shuffle4(workState, key, shuffle&0x0F ); // keep 4 LSBs
      printStateHex(workState);
      initState(state, workState);
      cipher_shuffle5(workState, key, shuffle&0x1F ); // keep 5 LSBs
      printStateHex(workState);
      initState(state, workState);
      cipher_shuffle6(workState, key, shuffle&0x3F ); // keep 6 LSBs
      printStateHex(workState);
      initState(state, workState);
      cipher_shuffle8(workState, key, shuffle&0xFF ); // keep 8 LSBs
      printStateHex(workState);
      initState(state, workState);
      cipher_shuffle9(workState, key, longShuffle&0x01FF ); // keep 9 LSBs
      printStateHex(workState);
      initState(state, workState);
      cipher_shuffle10(workState, key, longShuffle ); // keep 10 LSBs
      printStateHex(workState);
      initState(state, workState);
      
    }else if (byteIn == 'v'){
      //change values, keys, message, etc
      changeValue(state);
    }else if(byteIn == 'p'){
      // print hex value of random values
      printStateHex(state);
    }
  }
}

void changeValue(unsigned char state[][Ncols]){
  for(unsigned char i=0; i<Nrows;++i){
    for(unsigned char j=0; j<Ncols;++j){
      state[i][j] = random(256);
    }
  }
  shuffle = random(256);
  longShuffle = random(1024);
}

void initState(unsigned char state[][Ncols], unsigned char workState[][Ncols]){
  for(unsigned char i=0; i<Nrows;++i){
    for(unsigned char j=0; j<Ncols;++j){
      workState[i][j] = state[i][j];
    }
  }
}

void printStateHex(unsigned char state[][Ncols]){
  for(unsigned char i=0; i<Nrows;++i){
    for(unsigned char j=0; j<Ncols;++j){
      printHex(state[j][i]);
      Serial.print(", ");
    }
  }
  printHex(shuffle);
  Serial.println();
}

char tmp[2];
void printHex(unsigned char value){
  //Serial.print("0x");Serial.print(state[j][i], HEX);
  sprintf(tmp, "%02X", value);
  Serial.print("0x");
  Serial.print(tmp[0]);
  Serial.print(tmp[1]);
}

void speedTest(){
  // Speed tests:
  unsigned long startTime, time;
  unsigned long repetitions = 10000;
  
  Serial.println("Starting speed test for different versions of Random Start Index Matrix shuffle AES");
  Serial.println("Encryption time of "+String(repetitions) + " blocks (in milliseconds)");
  initState(state, workState);
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    subBytes(workState);
    //cipher(workState, key);
  }
  time = millis() - startTime;
  Serial.println("AES no shuffle:\t"+String(time));
  
  
  startTime = millis();
  
  for(unsigned i=0; i < repetitions; ++i){
    shuffle = random(2);
    subBytes_shuffle1(workState, shuffle);
    //cipher_shuffle1(workState, key, shuffle ); //keep 1 LSB
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 1:\t"+String(time));
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    shuffle = random(4);
    subBytes_shuffle2(workState, shuffle);
    //cipher_shuffle2(workState, key, shuffle ); // keep 2 LSBs
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 2:\t"+String(time));
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    shuffle = random(8);
    subBytes_shuffle3(workState, shuffle);
    //cipher_shuffle3(workState, key, shuffle ); // keep 3 LSBs
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 3:\t"+String(time));
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    shuffle = random(16);
    subBytes_shuffle4(workState, shuffle);
    //cipher_shuffle4(workState, key, shuffle ); // keep 4 LSBs
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 4:\t"+String(time));
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    shuffle = random(32);
    subBytes_shuffle5(workState, shuffle);
    //cipher_shuffle5(workState, key, shuffle ); 
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 5:\t"+String(time));
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    shuffle = random(64);
    subBytes_shuffle6(workState, shuffle);
    //cipher_shuffle6(workState, key, shuffle ); 
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 6:\t"+String(time));
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    shuffle = random(256);
    subBytes_shuffle8(workState, shuffle);
    //cipher_shuffle8(workState, key, shuffle ); 
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 8:\t"+String(time));
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    longShuffle = random(512);
    subBytes_shuffle9(workState, longShuffle);
    //cipher_shuffle9(workState, key, longShuffle ); 
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 9:\t"+String(time));
  
  startTime = millis();
  for(unsigned i=0; i < repetitions; ++i){
    longShuffle = random(1024);
    subBytes_shuffle10(workState, longShuffle);
    //cipher_shuffle10(workState, key, longShuffle );
  }
  time = millis() - startTime;
  Serial.println("AES shuffle 10:\t"+String(time));
}
